summaryrefslogtreecommitdiff
diff options
context:
space:
mode:
authorBram Moolenaar <Bram@vim.org>2021-02-12 21:32:47 +0100
committerBram Moolenaar <Bram@vim.org>2021-02-12 21:32:47 +0100
commitd9d7789b6fe5f2b4074375ee9f1c0bad3e4d3cfe (patch)
treec4f8094ae801dab2d4bed5e048ead94cb0922417
parentca753ec862f8191f1fc5ed161753704a488ed08d (diff)
downloadvim-git-d9d7789b6fe5f2b4074375ee9f1c0bad3e4d3cfe.tar.gz
patch 8.2.2503: Vim9: a caught error may leave something on the stackv8.2.2503
Problem: Vim9: a caught error may leave something on the stack. Solution: Drop items from the stack if needed. (closes #7826)
-rw-r--r--src/testdir/test_vim9_script.vim10
-rw-r--r--src/version.c2
-rw-r--r--src/vim9execute.c10
3 files changed, 21 insertions, 1 deletions
diff --git a/src/testdir/test_vim9_script.vim b/src/testdir/test_vim9_script.vim
index 42a0d61dd..17b996f17 100644
--- a/src/testdir/test_vim9_script.vim
+++ b/src/testdir/test_vim9_script.vim
@@ -556,6 +556,16 @@ def Test_try_catch_throw()
n = 411
endtry
assert_equal(411, n)
+
+ var counter = 0
+ for i in range(4)
+ try
+ eval [][0]
+ catch
+ endtry
+ counter += 1
+ endfor
+ assert_equal(4, counter)
enddef
def Test_cnext_works_in_catch()
diff --git a/src/version.c b/src/version.c
index ac9087a80..60f8a711d 100644
--- a/src/version.c
+++ b/src/version.c
@@ -751,6 +751,8 @@ static char *(features[]) =
static int included_patches[] =
{ /* Add new patch number below this line */
/**/
+ 2503,
+/**/
2502,
/**/
2501,
diff --git a/src/vim9execute.c b/src/vim9execute.c
index 6d462eaf1..8febf3623 100644
--- a/src/vim9execute.c
+++ b/src/vim9execute.c
@@ -24,7 +24,8 @@
// Structure put on ec_trystack when ISN_TRY is encountered.
typedef struct {
- int tcd_frame_idx; // ec_frame_idx when ISN_TRY was encountered
+ int tcd_frame_idx; // ec_frame_idx at ISN_TRY
+ int tcd_stack_len; // size of ectx.ec_stack at ISN_TRY
int tcd_catch_idx; // instruction of the first catch
int tcd_finally_idx; // instruction of the finally block
int tcd_caught; // catch block entered
@@ -2561,6 +2562,7 @@ call_def_function(
++ectx.ec_trystack.ga_len;
++trylevel;
trycmd->tcd_frame_idx = ectx.ec_frame_idx;
+ trycmd->tcd_stack_len = ectx.ec_stack.ga_len;
trycmd->tcd_catch_idx = iptr->isn_arg.try.try_catch;
trycmd->tcd_finally_idx = iptr->isn_arg.try.try_finally;
trycmd->tcd_caught = FALSE;
@@ -2632,6 +2634,12 @@ call_def_function(
if (trycmd->tcd_return)
goto func_return;
+
+ while (ectx.ec_stack.ga_len > trycmd->tcd_stack_len)
+ {
+ --ectx.ec_stack.ga_len;
+ clear_tv(STACK_TV_BOT(0));
+ }
}
}
break;